@@ -177,7 +177,8 @@ int main(int argc, char* argv[])
 	CK_C_GetFunctionList pGetFunctionList = loadLibrary(module, &moduleHandle, &errMsg);
 	if (pGetFunctionList == NULL)
 	{
-		fprintf(stderr, "ERROR: Could not load the library: %s\n", errMsg);
+		fprintf(stderr, "ERROR: Could not load the PKCS#11 library/module: %s\n", errMsg);
+		fprintf(stderr, "ERROR: Please check log files for additional information.\n");
 		exit(1);
 	}
 
@@ -188,7 +189,8 @@ int main(int argc, char* argv[])
 	rv = p11->C_Initialize(NULL_PTR);
 	if (rv != CKR_OK)
 	{
-		fprintf(stderr, "ERROR: Could not initialize the library.\n");
+		fprintf(stderr, "ERROR: Could not initialize the PKCS#11 library/module: %s\n", module ? module : DEFAULT_PKCS11_LIB);
+		fprintf(stderr, "ERROR: Please check log files for additional information.\n");
 		exit(1);
 	}
 
