@@ -396,7 +396,8 @@ enum AttributeKind {
 	akBoolean,
 	akInteger,
 	akBinary,
-	akArray
+	akAttrMap,
+	akMechSet
 };
 
 static AttributeKind attributeKind(CK_ATTRIBUTE_TYPE type)
@@ -505,10 +506,10 @@ static AttributeKind attributeKind(CK_ATTRIBUTE_TYPE type)
 	case CKA_DEFAULT_CMS_ATTRIBUTES:
 	case CKA_SUPPORTED_CMS_ATTRIBUTES:
 */
-	case CKA_WRAP_TEMPLATE: return akArray;
-	case CKA_UNWRAP_TEMPLATE: return akArray;
-	case CKA_DERIVE_TEMPLATE: return akArray;
-	case CKA_ALLOWED_MECHANISMS: return akArray;
+	case CKA_WRAP_TEMPLATE: return akAttrMap;
+	case CKA_UNWRAP_TEMPLATE: return akAttrMap;
+	case CKA_DERIVE_TEMPLATE: return akAttrMap;
+	case CKA_ALLOWED_MECHANISMS: return akMechSet;
 
 	case CKA_OS_TOKENLABEL: return akBinary;
 	case CKA_OS_TOKENSERIAL: return akBinary;
@@ -520,7 +521,39 @@ static AttributeKind attributeKind(CK_ATTRIBUTE_TYPE type)
 	}
 }
 
-static bool decodeArray(std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& array, const unsigned char *binary, size_t size)
+static bool decodeMechanismTypeSet(std::set<CK_MECHANISM_TYPE>& set, const unsigned char *binary, size_t size)
+{
+	for (size_t pos = 0; pos < size; )
+	{
+		// finished?
+		if (pos == size) break;
+
+		CK_MECHANISM_TYPE mechType;
+		if (pos + sizeof(mechType) > size)
+		{
+			ERROR_MSG("mechanism type set overrun");
+			return false;
+		}
+
+		memcpy(&mechType, binary + pos, sizeof(mechType));
+		pos += sizeof(mechType);
+
+		set.insert(mechType);
+    }
+
+	return true;
+}
+
+static void encodeMechanismTypeSet(ByteString& value, const std::set<CK_MECHANISM_TYPE>& set)
+{
+	for (std::set<CK_MECHANISM_TYPE>::const_iterator i = set.begin(); i != set.end(); ++i)
+	{
+		CK_MECHANISM_TYPE mechType = *i;
+		value += ByteString((unsigned char *) &mechType, sizeof(mechType));
+	}
+}
+
+static bool decodeAttributeMap(std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& map, const unsigned char *binary, size_t size)
 {
 	for (size_t pos = 0; pos < size; )
 	{
@@ -557,7 +590,7 @@ static bool decodeArray(std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& array, const un
 				memcpy(&value, binary + pos, sizeof(value));
 				pos += sizeof(value);
 
-				array.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
+				map.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
 			}
 			break;
 
@@ -571,7 +604,7 @@ static bool decodeArray(std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& array, const un
 				memcpy(&value, binary + pos, sizeof(value));
 				pos += sizeof(value);
 
-				array.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
+				map.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
 			}
 			break;
 
@@ -594,12 +627,37 @@ static bool decodeArray(std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& array, const un
 				memcpy(&value[0], binary + pos, len);
 				pos += len;
 
-				array.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
+				map.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
+			}
+			break;
+
+			case akMechSet:
+			{
+				unsigned long len;
+				if (pos + sizeof(len) > size)
+				{
+					goto overrun;
+				}
+				memcpy(&len, binary + pos, sizeof(len));
+				pos += sizeof(len);
+
+				if (pos + len > size)
+				{
+					goto overrun;
+				}
+
+				std::set<CK_MECHANISM_TYPE> value;
+				if (!decodeMechanismTypeSet(value, binary + pos, len)) {
+					return false;
+				}
+				pos += len;
+
+				map.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (attrType, value));
 			}
 			break;
 
 			default:
-			ERROR_MSG("unsupported attribute kind in array");
+			ERROR_MSG("unsupported attribute kind in attribute map");
 
 			return false;
 		}
@@ -608,12 +666,12 @@ static bool decodeArray(std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& array, const un
 	return true;
 
 overrun:
-	ERROR_MSG("array template overrun");
+	ERROR_MSG("attribute map template overrun");
 
 	return false;
 }
 
-static bool encodeArray(ByteString& value, const std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& attributes)
+static bool encodeAttributeMap(ByteString& value, const std::map<CK_ATTRIBUTE_TYPE,OSAttribute>& attributes)
 {
 	for (std::map<CK_ATTRIBUTE_TYPE,OSAttribute>::const_iterator i = attributes.begin(); i != attributes.end(); ++i)
 	{
@@ -647,9 +705,21 @@ static bool encodeArray(ByteString& value, const std::map<CK_ATTRIBUTE_TYPE,OSAt
 			value += ByteString((unsigned char*) &len, sizeof(len));
 			value += val;
 		}
+		else if (attr.isMechanismTypeSetAttribute())
+		{
+			AttributeKind attrKind = akMechSet;
+			value += ByteString((unsigned char*) &attrKind, sizeof(attrKind));
+
+			ByteString val;
+			encodeMechanismTypeSet(val, attr.getMechanismTypeSetValue());
+
+			unsigned long len = val.size();
+			value += ByteString((unsigned char*) &len, sizeof(len));
+			value += val;
+		}
 		else
 		{
-			ERROR_MSG("unsupported attribute kind for array");
+			ERROR_MSG("unsupported attribute kind for attribute map");
 
 			return false;
 		}
@@ -789,7 +859,57 @@ OSAttribute *DBObject::accessAttribute(CK_ATTRIBUTE_TYPE type)
 			}
 			return attr;
 		}
-		case akArray:
+		case akMechSet:
+		{
+			// try to find the attribute in the binary attribute table
+			DB::Statement statement = _connection->prepare(
+					"select value from attribute_binary where type=%lu and object_id=%lld",
+					type,
+					_objectId);
+			if (!statement.isValid())
+			{
+				return NULL;
+			}
+			DB::Result result = _connection->perform(statement);
+			if (!result.isValid())
+			{
+				return NULL;
+			}
+			// Store the attribute in the transaction when it is active.
+			std::map<CK_ATTRIBUTE_TYPE,OSAttribute*> *attrs = &_attributes;
+			if (_transaction)
+				attrs = _transaction;
+
+			const unsigned char *value = result.getBinary(1);
+			size_t size = result.getFieldLength(1);
+
+			std::set<CK_MECHANISM_TYPE> set;
+			if (!decodeMechanismTypeSet(set, value, size))
+			{
+				return NULL;
+			}
+
+			OSAttribute *attr;
+			std::map<CK_ATTRIBUTE_TYPE,OSAttribute*>::iterator it =	 attrs->find(type);
+			if (it != attrs->end())
+			{
+				if (it->second != NULL)
+				{
+					delete it->second;
+				}
+
+				it->second = new OSAttribute(set);
+				attr = it->second;
+			}
+			else
+			{
+				attr = new OSAttribute(set);
+				(*attrs)[type] = attr;
+				return attr;
+			}
+			return attr;
+		}
+		case akAttrMap:
 		{
 			// try to find the attribute in the array attribute table
 			DB::Statement statement = _connection->prepare(
@@ -817,7 +937,7 @@ OSAttribute *DBObject::accessAttribute(CK_ATTRIBUTE_TYPE type)
 			if (it != attrs->end())
 			{
 				std::map<CK_ATTRIBUTE_TYPE,OSAttribute> value;
-				if (!decodeArray(value,binary,size))
+				if (!decodeAttributeMap(value,binary,size))
 				{
 					return NULL;
 				}
@@ -833,7 +953,7 @@ OSAttribute *DBObject::accessAttribute(CK_ATTRIBUTE_TYPE type)
 			else
 			{
 				std::map<CK_ATTRIBUTE_TYPE,OSAttribute> value;
-				if (!decodeArray(value,binary,size))
+				if (!decodeAttributeMap(value,binary,size))
 				{
 					return NULL;
 				}
@@ -1002,7 +1122,6 @@ bool DBObject::setAttribute(CK_ATTRIBUTE_TYPE type, const OSAttribute& attribute
 	if (attr)
 	{
 		DB::Statement statement;
-		bool bindByteString = true;
 		if (attr->isBooleanAttribute())
 		{
 			// update boolean attribute
@@ -1011,7 +1130,6 @@ bool DBObject::setAttribute(CK_ATTRIBUTE_TYPE type, const OSAttribute& attribute
 					attribute.getBooleanValue() ? 1 : 0,
 					type,
 					_objectId);
-			bindByteString = false;
 		}
 		else if (attr->isUnsignedLongAttribute())
 		{
@@ -1021,7 +1139,6 @@ bool DBObject::setAttribute(CK_ATTRIBUTE_TYPE type, const OSAttribute& attribute
 					static_cast<long long>(attribute.getUnsignedLongValue()),
 					type,
 					_objectId);
-			bindByteString = false;
 		}
 		else if (attr->isByteStringAttribute())
 		{
@@ -1030,13 +1147,25 @@ bool DBObject::setAttribute(CK_ATTRIBUTE_TYPE type, const OSAttribute& attribute
 					"update attribute_binary set value=? where type=%lu and object_id=%lld",
 					type,
 					_objectId);
-			//bindByteString = true;
+			DB::Bindings(statement).bindBlob(1, attribute.getByteStringValue().const_byte_str(), attribute.getByteStringValue().size(), SQLITE_STATIC);
 		}
-		else if (attr->isArrayAttribute())
+		else if (attr->isMechanismTypeSetAttribute())
 		{
-			// update array attribute
+			// update binary attribute
 			ByteString value;
-			if (!encodeArray(value, attribute.getArrayValue()))
+			encodeMechanismTypeSet(value, attribute.getMechanismTypeSetValue());
+
+			statement = _connection->prepare(
+					"update attribute_binary set value=? where type=%lu and object_id=%lld",
+					type,
+					_objectId);
+			DB::Bindings(statement).bindBlob(1, value.const_byte_str(), value.size(), SQLITE_TRANSIENT);
+		}
+		else if (attr->isAttributeMapAttribute())
+		{
+			// update attribute map attribute
+			ByteString value;
+			if (!encodeAttributeMap(value, attribute.getAttributeMapValue()))
 			{
 				return false;
 			}
@@ -1046,12 +1175,6 @@ bool DBObject::setAttribute(CK_ATTRIBUTE_TYPE type, const OSAttribute& attribute
 					type,
 					_objectId);
 			DB::Bindings(statement).bindBlob(1, value.const_byte_str(), value.size(), SQLITE_TRANSIENT);
-			bindByteString = false;
-		}
-
-		if (bindByteString)
-		{
-			DB::Bindings(statement).bindBlob(1, attribute.getByteStringValue().const_byte_str(), attribute.getByteStringValue().size(), SQLITE_STATIC);
 		}
 
 		// Statement is valid when a prepared statement has been attached to it.
@@ -1108,11 +1231,23 @@ bool DBObject::setAttribute(CK_ATTRIBUTE_TYPE type, const OSAttribute& attribute
 
 		DB::Bindings(statement).bindBlob(1, attribute.getByteStringValue().const_byte_str(), attribute.getByteStringValue().size(), SQLITE_STATIC);
 	}
-	else if (attribute.isArrayAttribute())
+	else if (attribute.isMechanismTypeSetAttribute())
 	{
 		// Could not update it, so we need to insert it.
 		ByteString value;
-		if (!encodeArray(value, attribute.getArrayValue()))
+		encodeMechanismTypeSet(value, attribute.getMechanismTypeSetValue());
+
+		statement = _connection->prepare(
+				"insert into attribute_binary (value,type,object_id) values (?,%lu,%lld)",
+				type,
+				_objectId);
+		DB::Bindings(statement).bindBlob(1, value.const_byte_str(), value.size(), SQLITE_TRANSIENT);
+	}
+	else if (attribute.isAttributeMapAttribute())
+	{
+		// Could not update it, so we need to insert it.
+		ByteString value;
+		if (!encodeAttributeMap(value, attribute.getAttributeMapValue()))
 		{
 			return false;
 		}
@@ -1184,7 +1319,7 @@ bool DBObject::deleteAttribute(CK_ATTRIBUTE_TYPE type)
 				type,
 				_objectId);
 	}
-	else if (attr->isByteStringAttribute())
+	else if (attr->isByteStringAttribute() || attr -> isMechanismTypeSetAttribute())
 	{
 		// delete binary attribute
 		statement = _connection->prepare(
@@ -1192,9 +1327,9 @@ bool DBObject::deleteAttribute(CK_ATTRIBUTE_TYPE type)
 				type,
 				_objectId);
 	}
-	else if (attr->isArrayAttribute())
+	else if (attr->isAttributeMapAttribute())
 	{
-		// delete array attribute
+		// delete attribute map attribute
 		statement = _connection->prepare(
 				"delete from attribute_array where type=%lu and object_id=%lld",
 				type,
