@@ -289,7 +289,7 @@ void test_a_dbobject_with_an_object::should_store_binary_attributes()
 
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_MODULUS).getByteStringValue() == value1);
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_COEFFICIENT).getByteStringValue() == value2);
-		CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3); 
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3);
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_PUBLIC_EXPONENT).getByteStringValue() == value4);
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SUBJECT).getByteStringValue() == value5);
 
@@ -302,11 +302,50 @@ void test_a_dbobject_with_an_object::should_store_binary_attributes()
 	}
 }
 
-void test_a_dbobject_with_an_object::should_store_array_attributes()
+void test_a_dbobject_with_an_object::should_store_mechtypeset_attributes()
+{
+
+	// Create the test object
+	{
+		DBObject testObject(connection);
+		CPPUNIT_ASSERT(testObject.find(1));
+		CPPUNIT_ASSERT(testObject.isValid());
+
+		std::set<CK_MECHANISM_TYPE> set;
+		set.insert(CKM_SHA256);
+		set.insert(CKM_SHA512);
+		OSAttribute attr(set);
+
+		CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr));
+	}
+
+	// Now read back the object
+	{
+		DBObject testObject(connection);
+		CPPUNIT_ASSERT(testObject.find(1));
+		CPPUNIT_ASSERT(testObject.isValid());
+
+		CPPUNIT_ASSERT(testObject.attributeExists(CKA_ALLOWED_MECHANISMS));
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
+
+		std::set<CK_MECHANISM_TYPE> retrieved =
+				testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue();
+
+		CPPUNIT_ASSERT(retrieved.size() == 2);
+		CPPUNIT_ASSERT(retrieved.find(CKM_SHA256) != retrieved.end());
+		CPPUNIT_ASSERT(retrieved.find(CKM_SHA384) == retrieved.end());
+		CPPUNIT_ASSERT(retrieved.find(CKM_SHA512) != retrieved.end());
+	}
+}
+
+void test_a_dbobject_with_an_object::should_store_attrmap_attributes()
 {
 	bool value1 = true;
 	unsigned long value2 = 0x87654321;
 	ByteString value3 = "BDEBDBEDBBDBEBDEBE792759537328";
+	std::set<CK_MECHANISM_TYPE> value4;
+	value4.insert(CKM_SHA256);
+	value4.insert(CKM_SHA512);
 
 	// Create the test object
 	{
@@ -317,11 +356,13 @@ void test_a_dbobject_with_an_object::should_store_array_attributes()
 		OSAttribute attr1(value1);
 		OSAttribute attr2(value2);
 		OSAttribute attr3(value3);
+		OSAttribute attr4(value4);
 
 		std::map<CK_ATTRIBUTE_TYPE,OSAttribute> mattr;
 		mattr.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (CKA_TOKEN, attr1));
 		mattr.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (CKA_PRIME_BITS, attr2));
-		mattr.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (CKA_VALUE_BITS, attr3));
+		mattr.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (CKA_VALUE, attr3));
+		mattr.insert(std::pair<CK_ATTRIBUTE_TYPE,OSAttribute> (CKA_ALLOWED_MECHANISMS, attr4));
 		OSAttribute attra(mattr);
 
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_WRAP_TEMPLATE, attra));
@@ -336,17 +377,21 @@ void test_a_dbobject_with_an_object::should_store_array_attributes()
 		CPPUNIT_ASSERT(testObject.attributeExists(CKA_WRAP_TEMPLATE));
 		CPPUNIT_ASSERT(!testObject.attributeExists(CKA_UNWRAP_TEMPLATE));
 
-		std::map<CK_ATTRIBUTE_TYPE,OSAttribute> mattrb = testObject.getAttribute(CKA_WRAP_TEMPLATE).getArrayValue();
-		CPPUNIT_ASSERT(mattrb.size() == 3);
+		std::map<CK_ATTRIBUTE_TYPE,OSAttribute> mattrb =
+				testObject.getAttribute(CKA_WRAP_TEMPLATE).getAttributeMapValue();
+		CPPUNIT_ASSERT(mattrb.size() == 4);
 		CPPUNIT_ASSERT(mattrb.find(CKA_TOKEN) != mattrb.end());
 		CPPUNIT_ASSERT(mattrb.at(CKA_TOKEN).isBooleanAttribute());
 		CPPUNIT_ASSERT(mattrb.at(CKA_TOKEN).getBooleanValue() == true);
 		CPPUNIT_ASSERT(mattrb.find(CKA_PRIME_BITS) != mattrb.end());
 		CPPUNIT_ASSERT(mattrb.at(CKA_PRIME_BITS).isUnsignedLongAttribute());
 		CPPUNIT_ASSERT(mattrb.at(CKA_PRIME_BITS).getUnsignedLongValue() == 0x87654321);
-		CPPUNIT_ASSERT(mattrb.find(CKA_VALUE_BITS) != mattrb.end());
-		CPPUNIT_ASSERT(mattrb.at(CKA_VALUE_BITS).isByteStringAttribute());
-		CPPUNIT_ASSERT(mattrb.at(CKA_VALUE_BITS).getByteStringValue() == value3);
+		CPPUNIT_ASSERT(mattrb.find(CKA_VALUE) != mattrb.end());
+		CPPUNIT_ASSERT(mattrb.at(CKA_VALUE).isByteStringAttribute());
+		CPPUNIT_ASSERT(mattrb.at(CKA_VALUE).getByteStringValue() == value3);
+		CPPUNIT_ASSERT(mattrb.find(CKA_ALLOWED_MECHANISMS) != mattrb.end());
+		CPPUNIT_ASSERT(mattrb.at(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
+		CPPUNIT_ASSERT(mattrb.at(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value4);
 	}
 }
 
@@ -355,6 +400,9 @@ void test_a_dbobject_with_an_object::should_store_mixed_attributes()
 	bool value1 = true;
 	unsigned long value2 = 0x87654321;
 	unsigned long value3 = 0xBDEBDBED;
+	std::set<CK_MECHANISM_TYPE> value4;
+	value4.insert(CKM_SHA256);
+	value4.insert(CKM_SHA512);
 
 	// Create the test object
 	{
@@ -365,10 +413,12 @@ void test_a_dbobject_with_an_object::should_store_mixed_attributes()
 		OSAttribute attr1(value1);
 		OSAttribute attr2(value2);
 		OSAttribute attr3(value3);
+		OSAttribute attr4(value4);
 
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_TOKEN, attr1));
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_PRIME_BITS, attr2));
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_VALUE_BITS, attr3));
+		CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr4));
 	}
 
 	// Now read back the object
@@ -380,15 +430,18 @@ void test_a_dbobject_with_an_object::should_store_mixed_attributes()
 		CPPUNIT_ASSERT(testObject.attributeExists(CKA_TOKEN));
 		CPPUNIT_ASSERT(testObject.attributeExists(CKA_PRIME_BITS));
 		CPPUNIT_ASSERT(testObject.attributeExists(CKA_VALUE_BITS));
+		CPPUNIT_ASSERT(testObject.attributeExists(CKA_ALLOWED_MECHANISMS));
 		CPPUNIT_ASSERT(!testObject.attributeExists(CKA_ID));
 
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).isBooleanAttribute());
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).getBooleanValue());
 		CPPUNIT_ASSERT_EQUAL(testObject.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue(), value2);
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3);
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value4);
 	}
 }
 
@@ -456,6 +509,12 @@ void test_a_dbobject_with_an_object::can_refresh_attributes()
 	ByteString value2 = "BDEBDBEDBBDBEBDEBE792759537328";
 	ByteString value2a = "466487346943785684957634";
 	ByteString value3 = "0102010201020102010201020102010201020102";
+	std::set<CK_MECHANISM_TYPE> value4;
+	value4.insert(CKM_SHA256);
+	value4.insert(CKM_SHA512);
+	std::set<CK_MECHANISM_TYPE> value4a;
+	value4a.insert(CKM_SHA384);
+	value4a.insert(CKM_SHA512);
 
 	// Create the test object
 	{
@@ -465,9 +524,11 @@ void test_a_dbobject_with_an_object::can_refresh_attributes()
 
 		OSAttribute attr1(value1);
 		OSAttribute attr2(value2);
+		OSAttribute attr4(value4);
 
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_SIGN, attr1));
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_SUBJECT, attr2));
+		CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr4));
 	}
 
 	// Now read back the object
@@ -478,29 +539,36 @@ void test_a_dbobject_with_an_object::can_refresh_attributes()
 
 		CPPUNIT_ASSERT(testObject.attributeExists(CKA_SIGN));
 		CPPUNIT_ASSERT(testObject.attributeExists(CKA_SUBJECT));
+		CPPUNIT_ASSERT(testObject.attributeExists(CKA_ALLOWED_MECHANISMS));
 		CPPUNIT_ASSERT(!testObject.attributeExists(CKA_ID));
 
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SIGN).isBooleanAttribute());
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SUBJECT).isByteStringAttribute());
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SIGN).getBooleanValue());
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SUBJECT).getByteStringValue() == value2);
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value4);
 
 		OSAttribute attr1(value1a);
 		OSAttribute attr2(value2a);
+		OSAttribute attr4(value4a);
 
 		// Change the attributes
 		CPPUNIT_ASSERT(testObject.isValid());
 
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_SIGN, attr1));
 		CPPUNIT_ASSERT(testObject.setAttribute(CKA_SUBJECT, attr2));
+		CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr4));
 
 		// Check the attributes
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SIGN).isBooleanAttribute());
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SUBJECT).isByteStringAttribute());
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SIGN).getBooleanValue() == value1a);
 		CPPUNIT_ASSERT(testObject.getAttribute(CKA_SUBJECT).getByteStringValue() == value2a);
+		CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value4a);
 
 		// Open the object a second time
 		DBObject testObject2(connection);
@@ -510,9 +578,11 @@ void test_a_dbobject_with_an_object::can_refresh_attributes()
 		// Check the attributes on the second instance
 		CPPUNIT_ASSERT(testObject2.getAttribute(CKA_SIGN).isBooleanAttribute());
 		CPPUNIT_ASSERT(testObject2.getAttribute(CKA_SUBJECT).isByteStringAttribute());
+		CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 		CPPUNIT_ASSERT(testObject2.getAttribute(CKA_SIGN).getBooleanValue() == value1a);
 		CPPUNIT_ASSERT(testObject2.getAttribute(CKA_SUBJECT).getByteStringValue() == value2a);
+		CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value4a);
 
 		// Add an attribute on the second object
 		OSAttribute attr3(value3);
@@ -552,16 +622,21 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	unsigned long value2 = 0x87654321;
 	unsigned long value3 = 0xBDEBDBED;
 	ByteString value4 = "AAAAAAAAAAAAAAAFFFFFFFFFFFFFFF";
+	std::set<CK_MECHANISM_TYPE> value5;
+	value5.insert(CKM_SHA256);
+	value5.insert(CKM_SHA512);
 
 	OSAttribute attr1(value1);
 	OSAttribute attr2(value2);
 	OSAttribute attr3(value3);
 	OSAttribute attr4(value4);
+	OSAttribute attr5(value5);
 
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_TOKEN, attr1));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_PRIME_BITS, attr2));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_VALUE_BITS, attr3));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_ID, attr4));
+	CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr5));
 
 	// Create secondary instance for the same object.
 	// This needs to have a different connection to the database to simulate
@@ -575,23 +650,29 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	// Check that the attributes have the same values as set on testObject.
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_TOKEN).getBooleanValue() == value1);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ID).getByteStringValue() == value4);
+	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5);
 
 	// New values
 	bool value1a = false;
 	unsigned long value2a = 0x12345678;
 	unsigned long value3a = 0xABABABAB;
 	ByteString value4a = "EDEDEDEDEDEDEDEDEDEDEDEDEDEDED";
+	std::set<CK_MECHANISM_TYPE> value5a;
+	value5a.insert(CKM_SHA384);
+	value5a.insert(CKM_SHA512);
 
 	OSAttribute attr1a(value1a);
 	OSAttribute attr2a(value2a);
 	OSAttribute attr3a(value3a);
 	OSAttribute attr4a(value4a);
+	OSAttribute attr5a(value5a);
 
 	// Start transaction on object
 	CPPUNIT_ASSERT(testObject.startTransaction(DBObject::ReadWrite));
@@ -601,28 +682,33 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_PRIME_BITS, attr2a));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_VALUE_BITS, attr3a));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_ID, attr4a));
+	CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr5a));
 
 	// Verify that the attributes were set
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).isBooleanAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).getBooleanValue() == value1a);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2a);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3a);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ID).getByteStringValue() == value4a);
+	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5a);
 
 	// Verify that they are unchanged on the other instance
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_TOKEN).isBooleanAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_TOKEN).getBooleanValue() == value1);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ID).getByteStringValue() == value4);
+	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5);
 
 	// Commit the transaction
 	CPPUNIT_ASSERT(testObject.commitTransaction());
@@ -633,11 +719,13 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	// NOTE: 3 attributes below cannot be modified after creation and therefore are not required to propagate.
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_TOKEN).getBooleanValue() != value1a);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() != value2a);
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() != value3a);
+	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() != value5a);
 
 	// CKA_ID attribute can be modified after creation and therefore should have propagated.
 	CPPUNIT_ASSERT(testObject2.getAttribute(CKA_ID).getByteStringValue() == value4a);
@@ -650,17 +738,20 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_PRIME_BITS, attr2));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_VALUE_BITS, attr3));
 	CPPUNIT_ASSERT(testObject.setAttribute(CKA_ID, attr4));
+	CPPUNIT_ASSERT(testObject.setAttribute(CKA_ALLOWED_MECHANISMS, attr5));
 
 	// Verify that the attributes were set
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).isBooleanAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).getBooleanValue() == value1);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ID).getByteStringValue() == value4);
+	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5);
 
 	// Create a fresh third instance for the same object to force the data to be retrieved from the database.
 	DBObject testObject3(connection2);
@@ -672,12 +763,14 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	// Verify that the attributes from the database are still hodling the same value as when the transaction started.
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_TOKEN).getBooleanValue() == value1a);
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2a);
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3a);
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ID).getByteStringValue() == value4a);
+	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5a);
 
 	// Abort the transaction
 	CPPUNIT_ASSERT(testObject.abortTransaction());
@@ -688,24 +781,28 @@ void test_a_dbobject_with_an_object::should_use_transactions()
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	// After aborting a transaction the testObject should be back to pre transaction state.
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_TOKEN).getBooleanValue() == value1a);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2a);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3a);
 	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ID).getByteStringValue() == value4a);
+	CPPUNIT_ASSERT(testObject.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5a);
 
 	// Verify that testObject3 still has the original values.
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_TOKEN).isBooleanAttribute());
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_PRIME_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_VALUE_BITS).isUnsignedLongAttribute());
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ID).isByteStringAttribute());
+	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ALLOWED_MECHANISMS).isMechanismTypeSetAttribute());
 
 	// Verify that testObject3 still has the original values.
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_TOKEN).getBooleanValue() == value1a);
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_PRIME_BITS).getUnsignedLongValue() == value2a);
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_VALUE_BITS).getUnsignedLongValue() == value3a);
 	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ID).getByteStringValue() == value4a);
+	CPPUNIT_ASSERT(testObject3.getAttribute(CKA_ALLOWED_MECHANISMS).getMechanismTypeSetValue() == value5a);
 }
 
 void test_a_dbobject_with_an_object::should_fail_to_delete()
