@@ -195,6 +195,44 @@ void SignVerifyTests::signVerifySingle(CK_MECHANISM_TYPE mechanismType, CK_SESSI
 	CPPUNIT_ASSERT(rv==CKR_SIGNATURE_INVALID);
 }
 
+void SignVerifyTests::signVerifySingleData(size_t dataSize, CK_MECHANISM_TYPE mechanismType, CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hPublicKey, CK_OBJECT_HANDLE hPrivateKey, CK_VOID_PTR param /* = NULL_PTR */, CK_ULONG paramLen /* = 0 */)
+{
+	CK_RV rv;
+	CK_MECHANISM mechanism = { mechanismType, param, paramLen };
+	CK_BYTE *data = (CK_BYTE*)malloc(dataSize);
+	CK_BYTE signature[1024];
+	CK_ULONG ulSignatureLen = 0;
+	unsigned i;
+
+	CPPUNIT_ASSERT(data != NULL);
+
+	for (i=0;i<dataSize;i++)
+		data[i] = i;
+
+	rv = CRYPTOKI_F_PTR( C_SignInit(hSession,&mechanism,hPrivateKey) );
+	CPPUNIT_ASSERT(rv==CKR_OK);
+
+	ulSignatureLen = sizeof(signature);
+	rv = CRYPTOKI_F_PTR( C_Sign(hSession,data,dataSize,signature,&ulSignatureLen) );
+	CPPUNIT_ASSERT(rv==CKR_OK);
+
+	rv = CRYPTOKI_F_PTR( C_VerifyInit(hSession,&mechanism,hPublicKey) );
+	CPPUNIT_ASSERT(rv==CKR_OK);
+
+	rv = CRYPTOKI_F_PTR( C_Verify(hSession,data,dataSize,signature,ulSignatureLen) );
+	CPPUNIT_ASSERT(rv==CKR_OK);
+
+	// verify again, but now change the input that is being signed.
+	rv = CRYPTOKI_F_PTR( C_VerifyInit(hSession,&mechanism,hPublicKey) );
+	CPPUNIT_ASSERT(rv==CKR_OK);
+
+	data[0] = 0xff;
+	rv = CRYPTOKI_F_PTR( C_Verify(hSession,data,dataSize,signature,ulSignatureLen) );
+	CPPUNIT_ASSERT(rv==CKR_SIGNATURE_INVALID);
+
+	free(data);
+}
+
 void SignVerifyTests::signVerifyMulti(CK_MECHANISM_TYPE mechanismType, CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hPublicKey, CK_OBJECT_HANDLE hPrivateKey, CK_VOID_PTR param /* = NULL_PTR */, CK_ULONG paramLen /* = 0 */)
 {
 	CK_RV rv;
@@ -287,6 +325,15 @@ void SignVerifyTests::testRsaSignVerify()
 	signVerifyMulti(CKM_SHA256_RSA_PKCS, hSessionRO, hPuk,hPrk);
 	signVerifyMulti(CKM_SHA384_RSA_PKCS, hSessionRO, hPuk,hPrk);
 	signVerifyMulti(CKM_SHA512_RSA_PKCS, hSessionRO, hPuk,hPrk);
+
+#ifdef WITH_RAW_PSS
+	signVerifySingleData(20, CKM_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[0], sizeof(params[0]));
+	signVerifySingleData(28, CKM_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[1], sizeof(params[1]));
+	signVerifySingleData(32, CKM_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[2], sizeof(params[2]));
+	signVerifySingleData(48, CKM_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[3], sizeof(params[3]));
+	signVerifySingleData(64, CKM_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[4], sizeof(params[4]));
+#endif
+
 	signVerifyMulti(CKM_SHA1_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[0], sizeof(params[0]));
 	signVerifyMulti(CKM_SHA224_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[1], sizeof(params[1]));
 	signVerifyMulti(CKM_SHA256_RSA_PKCS_PSS, hSessionRO, hPuk,hPrk, &params[2], sizeof(params[2]));
@@ -438,7 +485,7 @@ CK_RV SignVerifyTests::generateKey(CK_SESSION_HANDLE hSession, CK_KEY_TYPE keyTy
 #ifndef WITH_BOTAN
 #define GEN_KEY_LEN	75
 #else
-#define GEN_KEY_LEN	55
+#define GEN_KEY_LEN	64
 #endif
 	CK_RV rv;
 	CK_OBJECT_CLASS keyClass = CKO_SECRET_KEY;
@@ -472,7 +519,66 @@ CK_RV SignVerifyTests::generateKey(CK_SESSION_HANDLE hSession, CK_KEY_TYPE keyTy
 	}
 }
 
-void SignVerifyTests::hmacSignVerify(CK_MECHANISM_TYPE mechanismType, CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hKey)
+CK_RV SignVerifyTests::generateDes2Key(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, CK_BBOOL bPrivate, CK_OBJECT_HANDLE &hKey)
+{
+	CK_MECHANISM mechanism = { CKM_DES2_KEY_GEN, NULL_PTR, 0 };
+	// CK_BBOOL bFalse = CK_FALSE;
+	CK_BBOOL bTrue = CK_TRUE;
+	CK_ATTRIBUTE keyAttribs[] = {
+		{ CKA_TOKEN, &bToken, sizeof(bToken) },
+		{ CKA_PRIVATE, &bPrivate, sizeof(bPrivate) },
+		{ CKA_SENSITIVE, &bTrue, sizeof(bTrue) },
+		{ CKA_VERIFY, &bTrue, sizeof(bTrue) },
+		{ CKA_SIGN, &bTrue, sizeof(bTrue) }
+	};
+
+	hKey = CK_INVALID_HANDLE;
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
+			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
+			     &hKey) );
+}
+
+CK_RV SignVerifyTests::generateDes3Key(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, CK_BBOOL bPrivate, CK_OBJECT_HANDLE &hKey)
+{
+	CK_MECHANISM mechanism = { CKM_DES3_KEY_GEN, NULL_PTR, 0 };
+	// CK_BBOOL bFalse = CK_FALSE;
+	CK_BBOOL bTrue = CK_TRUE;
+	CK_ATTRIBUTE keyAttribs[] = {
+		{ CKA_TOKEN, &bToken, sizeof(bToken) },
+		{ CKA_PRIVATE, &bPrivate, sizeof(bPrivate) },
+		{ CKA_SENSITIVE, &bTrue, sizeof(bTrue) },
+		{ CKA_VERIFY, &bTrue, sizeof(bTrue) },
+		{ CKA_SIGN, &bTrue, sizeof(bTrue) }
+	};
+
+	hKey = CK_INVALID_HANDLE;
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
+			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
+			     &hKey) );
+}
+
+CK_RV SignVerifyTests::generateAesKey(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, CK_BBOOL bPrivate, CK_OBJECT_HANDLE &hKey)
+{
+	CK_MECHANISM mechanism = { CKM_AES_KEY_GEN, NULL_PTR, 0 };
+	CK_ULONG bytes = 16;
+	// CK_BBOOL bFalse = CK_FALSE;
+	CK_BBOOL bTrue = CK_TRUE;
+	CK_ATTRIBUTE keyAttribs[] = {
+		{ CKA_TOKEN, &bToken, sizeof(bToken) },
+		{ CKA_PRIVATE, &bPrivate, sizeof(bPrivate) },
+		{ CKA_SENSITIVE, &bTrue, sizeof(bTrue) },
+		{ CKA_VERIFY, &bTrue, sizeof(bTrue) },
+		{ CKA_SIGN, &bTrue, sizeof(bTrue) },
+		{ CKA_VALUE_LEN, &bytes, sizeof(bytes) }
+	};
+
+	hKey = CK_INVALID_HANDLE;
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
+			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
+			     &hKey) );
+}
+
+void SignVerifyTests::macSignVerify(CK_MECHANISM_TYPE mechanismType, CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hKey)
 {
 	CK_RV rv;
 	CK_MECHANISM mechanism = { mechanismType, NULL_PTR, 0 };
@@ -511,7 +617,7 @@ void SignVerifyTests::hmacSignVerify(CK_MECHANISM_TYPE mechanismType, CK_SESSION
 	CPPUNIT_ASSERT(rv==CKR_SIGNATURE_INVALID);
 }
 
-void SignVerifyTests::testHmacSignVerify()
+void SignVerifyTests::testMacSignVerify()
 {
 	CK_RV rv;
 	CK_SESSION_HANDLE hSessionRO;
@@ -545,132 +651,180 @@ void SignVerifyTests::testHmacSignVerify()
 #ifndef WITH_FIPS
 	rv = generateKey(hSessionRW,CKK_MD5_HMAC,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_MD5_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_MD5_HMAC, hSessionRO, hKey);
 #endif
 
 	rv = generateKey(hSessionRW,CKK_SHA_1_HMAC,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA_1_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_SHA_1_HMAC, hSessionRO, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA224_HMAC,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA224_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_SHA224_HMAC, hSessionRO, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA256_HMAC,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA256_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_SHA256_HMAC, hSessionRO, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA384_HMAC,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA384_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_SHA384_HMAC, hSessionRO, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA512_HMAC,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA512_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_SHA512_HMAC, hSessionRO, hKey);
 
 #ifdef WITH_GOST
 	rv = generateKey(hSessionRW,CKK_GOST28147,IN_SESSION,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_GOSTR3411_HMAC, hSessionRO, hKey);
+	macSignVerify(CKM_GOSTR3411_HMAC, hSessionRO, hKey);
 #endif
 
+	rv = generateDes2Key(hSessionRW,IN_SESSION,IS_PUBLIC,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateDes3Key(hSessionRW,IN_SESSION,IS_PUBLIC,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateAesKey(hSessionRW,IN_SESSION,IS_PUBLIC,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_AES_CMAC, hSessionRO, hKey);
+
 	// Private Session Keys
 #ifndef WITH_FIPS
 	rv = generateKey(hSessionRW,CKK_MD5_HMAC,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_MD5_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_MD5_HMAC, hSessionRW, hKey);
 #endif
 
 	rv = generateKey(hSessionRW,CKK_SHA_1_HMAC,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA_1_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA_1_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA224_HMAC,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA224_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA256_HMAC,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA384_HMAC,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA384_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA384_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA512_HMAC,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA512_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA512_HMAC, hSessionRW, hKey);
 
 #ifdef WITH_GOST
 	rv = generateKey(hSessionRW,CKK_GOST28147,IN_SESSION,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_GOSTR3411_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_GOSTR3411_HMAC, hSessionRW, hKey);
 #endif
 
+	rv = generateDes2Key(hSessionRW,IN_SESSION,IS_PRIVATE,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateDes3Key(hSessionRW,IN_SESSION,IS_PRIVATE,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateAesKey(hSessionRW,IN_SESSION,IS_PRIVATE,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_AES_CMAC, hSessionRO, hKey);
+
 	// Public Token Keys
 #ifndef WITH_FIPS
 	rv = generateKey(hSessionRW,CKK_MD5_HMAC,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_MD5_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_MD5_HMAC, hSessionRW, hKey);
 #endif
 
 	rv = generateKey(hSessionRW,CKK_SHA_1_HMAC,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA_1_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA_1_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA224_HMAC,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA224_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA224_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA256_HMAC,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA384_HMAC,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA384_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA384_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA512_HMAC,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA512_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA512_HMAC, hSessionRW, hKey);
 
 #ifdef WITH_GOST
 	rv = generateKey(hSessionRW,CKK_GOST28147,ON_TOKEN,IS_PUBLIC,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_GOSTR3411_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_GOSTR3411_HMAC, hSessionRW, hKey);
 #endif
 
+	rv = generateDes2Key(hSessionRW,ON_TOKEN,IS_PUBLIC,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateDes3Key(hSessionRW,ON_TOKEN,IS_PUBLIC,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateAesKey(hSessionRW,ON_TOKEN,IS_PUBLIC,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_AES_CMAC, hSessionRO, hKey);
+
 	// Private Token Keys
 #ifndef WITH_FIPS
 	rv = generateKey(hSessionRW,CKK_MD5_HMAC,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_MD5_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_MD5_HMAC, hSessionRW, hKey);
 #endif
 
 	rv = generateKey(hSessionRW,CKK_SHA_1_HMAC,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA_1_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA_1_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA224_HMAC,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA224_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA224_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA256_HMAC,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA256_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA384_HMAC,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA384_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA384_HMAC, hSessionRW, hKey);
 
 	rv = generateKey(hSessionRW,CKK_SHA512_HMAC,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_SHA512_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_SHA512_HMAC, hSessionRW, hKey);
 
 #ifdef WITH_GOST
 	rv = generateKey(hSessionRW,CKK_GOST28147,ON_TOKEN,IS_PRIVATE,hKey);
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	hmacSignVerify(CKM_GOSTR3411_HMAC, hSessionRW, hKey);
+	macSignVerify(CKM_GOSTR3411_HMAC, hSessionRW, hKey);
 #endif
+
+	rv = generateDes2Key(hSessionRW,ON_TOKEN,IS_PRIVATE,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateDes3Key(hSessionRW,ON_TOKEN,IS_PRIVATE,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_DES3_CMAC, hSessionRO, hKey);
+
+	rv = generateAesKey(hSessionRW,ON_TOKEN,IS_PRIVATE,hKey);
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	macSignVerify(CKM_AES_CMAC, hSessionRO, hKey);
 }
 
