@@ -39,6 +39,7 @@ testlist = ["botan",
             "gost",
             "ossl",
             "osslv",
+            "rawpss",
             "rfc3394",
             "rfc5649"]
 
@@ -58,17 +59,20 @@ varnames = ["CUINCPATH",
             "LIBPATH",
             "PLATFORM",
             "PLATFORMDIR",
-            "PLATFORMTOOLSET"]
+            "PLATFORMTOOLSET",
+            "RUNTIMELIBRARY"]
 
 # conditions to stack
 
 condvals = {}
 
-condnames = ["BOTAN",
+condnames = ["AESGCM",
+             "BOTAN",
              "ECC",
              "GOST",
              "NONPAGE",
              "OPENSSL",
+             "RAWPSS",
              "RFC3394",
              "RFC5649",
              "TESTS"]
@@ -81,6 +85,7 @@ enablelist = ["64bit",
               "gost",
               "keep",
               "non-paged-memory",
+              "static-runtime",
               "verbose"]
 
 # with-xxx/without-xxx arguments
@@ -118,6 +123,7 @@ usage + [\
 "  enable-debug             enable build of Debug config [default=yes]",
 "  enable-ecc               enable support for ECC [default=yes]",
 "  enable-gost              enable support for GOST [default=yes]",
+"  enable-static-runtime    enable build with static CRT (/MT) [default=no]",
 "  enable-non-paged-memory  enable non-paged memory [default=yes]",
 "\nOptional Packages:",
 "  with-crypto-backend      select the crypto backend [openssl|botan]",
@@ -141,6 +147,7 @@ enable_keep = False
 enable_debug = True
 enable_ecc = True
 enable_gost = True
+enable_static_runtime = False
 enable_non_paged = True
 platform = 32
 crypto_backend = "openssl"
@@ -155,7 +162,7 @@ toolset = ""
 def dodetectplatform(visualstudio):
     # detect platform tool set >= VS2010
     global toolset
-    
+
     if "Microsoft Visual Studio 10.0" in visualstudio:
         toolset="v100"
     elif "Microsoft Visual Studio 11.0" in visualstudio:
@@ -240,7 +247,7 @@ def appargs(arg):
     global configargs
     # escape backslashes, spaces and double quotes
     escaped = ""
-    
+
     for x in arg:
         if (x == "\\") or (x == " ") or (x == "\""):
             escaped += "\\"
@@ -256,6 +263,7 @@ def myenable(key, val):
     global enable_debug
     global enable_ecc
     global enable_gost
+    global enable_static_runtime
     global enable_non_paged
     global enable_keep
     global verbose
@@ -277,6 +285,10 @@ def myenable(key, val):
         if not val:
             enable_gost = False
         return
+    if key.lower() == "static-runtime":
+        if val:
+            enable_static_runtime = True
+        return
     if key.lower() == "non-paged-memory":
         if not val:
             enable_non_paged = False
@@ -362,7 +374,7 @@ def mywith(key, val, detail=None):
             want_tests = False
             return
         if detail:
-            toolset=detail.lower() 
+            toolset=detail.lower()
         return
     want_unknown = True
     if not val:
@@ -395,6 +407,8 @@ def docleantest():
         os.unlink("botan.dll")
     if os.path.isfile("libeay32.dll"):
         os.unlink("libeay32.dll")
+    if os.path.isfile("libeaycompat32.dll"):
+        os.unlink("libeaycompat32.dll")
     if os.path.isfile("libcrypto-1_1.dll"):
         os.unlink("libcrypto-1_1.dll")
     if os.path.isfile("libcrypto-1_1-x64.dll"):
@@ -428,6 +442,12 @@ def doconfig():
         varvals["PLATFORM"] = "x64"
         varvals["PLATFORMDIR"] = "x64\\"
 
+    # configure the runtime library
+    if enable_static_runtime:
+        varvals["RUNTIMELIBRARY"] = "MultiThreaded"
+    else:
+        varvals["RUNTIMELIBRARY"] = "MultiThreadedDLL"
+
     # configure ECC and GOST
     if enable_ecc:
         condvals["ECC"] = True
@@ -439,31 +459,50 @@ def doconfig():
         condvals["BOTAN"] = True
         varvals["LIBNAME"] = "botan.lib"
         botan_path = os.path.abspath(botan_path)
-        botan_dll = os.path.join(botan_path, "botan.dll")
-        varvals["DLLPATH"] = botan_dll
         botan_inc = os.path.join(botan_path, "include")
-        if not os.path.exists(os.path.join(botan_inc, "botan\\init.h")):
+        botan_dll = ""
+        if os.path.exists(os.path.join(botan_inc, "botan-2\\botan\\init.h")):
+            varvals["INCLUDEPATH"] = os.path.join(botan_inc, "botan-2")
+        elif os.path.exists(os.path.join(botan_inc, "botan-1.11\\botan\\init.h")):
+            varvals["INCLUDEPATH"] = os.path.join(botan_inc, "botan-1.11")
+        elif os.path.exists(os.path.join(botan_inc, "botan\\init.h")):
+            varvals["INCLUDEPATH"] = botan_inc
+        else:
             print("can't find Botan includes", file=sys.stderr)
             sys.exit(1)
-        varvals["INCLUDEPATH"] = botan_inc
-        if not os.path.exists(os.path.join(botan_path, "botan.lib")):
+        if os.path.exists(os.path.join(botan_path, "lib\\botan.lib")):
+            varvals["LIBPATH"] = os.path.join(botan_path, "lib")
+            botan_dll = os.path.join(botan_path, "lib\\botan.dll")
+        elif os.path.exists(os.path.join(botan_path, "botan.lib")):
+            varvals["LIBPATH"] = botan_path
+            botan_dll = os.path.join(botan_path, "botan.dll")
+        else:
             print("can't find Botan library", file=sys.stderr)
             sys.exit(1)
-        varvals["LIBPATH"] = botan_path
+        varvals["DLLPATH"] = botan_dll
         if enable_debug:
             debug_botan_path = os.path.abspath(debug_botan_path)
-            varvals["DEBUGDLLPATH"] = \
-                os.path.join(debug_botan_path, "botan.dll")
             debug_botan_inc = os.path.join(debug_botan_path, "include")
-            if not os.path.exists(os.path.join(debug_botan_inc,
-                                               "botan\\init.h")):
+            debug_botan_dll = ""
+            if os.path.exists(os.path.join(debug_botan_inc, "botan-2\\botan\\init.h")):
+                varvals["DEBUGINCPATH"] = os.path.join(debug_botan_inc, "botan-2")
+            elif os.path.exists(os.path.join(debug_botan_inc, "botan-1.11\\botan\\init.h")):
+                varvals["DEBUGINCPATH"] = os.path.join(debug_botan_inc, "botan-1.11")
+            elif os.path.exists(os.path.join(debug_botan_inc, "botan\\init.h")):
+                varvals["DEBUGINCPATH"] = debug_botan_inc
+            else:
                 print("can't find debug Botan includes", file=sys.stderr)
                 sys.exit(1)
-            varvals["DEBUGINCPATH"] = debug_botan_inc
-            if not os.path.exists(os.path.join(debug_botan_path, "botan.lib")):
+            if os.path.exists(os.path.join(debug_botan_path, "lib\\botan.lib")):
+                varvals["DEBUGLIBPATH"] = os.path.join(debug_botan_path, "lib")
+                debug_botan_dll = os.path.join(debug_botan_path, "lib\\botan.dll")
+            if os.path.exists(os.path.join(debug_botan_path, "botan.lib")):
+                varvals["DEBUGLIBPATH"] = debug_botan_path
+                debug_botan_dll = os.path.join(debug_botan_path, "botan.dll")
+            else:
                 print("can't find debug Botan library", file=sys.stderr)
                 sys.exit(1)
-            varvals["DEBUGLIBPATH"] = debug_botan_path
+            varvals["DEBUGDLLPATH"] = debug_botan_dll
         else:
             varvals["DEBUGDLLPATH"] = varvals["DLLPATH"]
             varvals["DEBUGINCPATH"] = varvals["INCLUDEPATH"]
@@ -472,28 +511,28 @@ def doconfig():
         # Botan version
         if verbose:
             print("checking Botan version")
+        botan_version_major = 0
         botan_version_minor = 0
         system_libs = []
         if os.path.exists(botan_dll):
             subprocess.call(["copy", botan_dll, "."], shell=True)
         else:
             system_libs = ["user32.lib", "advapi32.lib"]
-        inc = botan_inc
-        lib = os.path.join(botan_path, "botan.lib")
+        inc = varvals["INCLUDEPATH"]
+        lib = os.path.join(varvals["LIBPATH"], "botan.lib")
         testfile = open("testbotan.cpp", "w")
         print('\
-#include <botan/init.h>\n\
 #include <botan/version.h>\n\
 int main() {\n\
- using namespace Botan;\n\
- LibraryInitializer::initialize();\n\
-#if BOTAN_VERSION_CODE < BOTAN_VERSION_CODE_FOR(1,10,0)\n\
- return 1;\n\
-#endif\n\
-#if BOTAN_VERSION_CODE > BOTAN_VERSION_CODE_FOR(1,11,0)\n\
+#if BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(2,0,0)\n\
+ return 3;\n\
+#elif BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(1,11,0)\n\
  return 2;\n\
-#endif\n\
+#elif BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(1,10,0)\n\
+ return 1;\n\
+#else\n\
  return 0;\n\
+#endif\n\
 }', file=testfile)
         testfile.close()
         command = ["cl", "/nologo", "/MD", "/I", inc, "testbotan.cpp", lib]
@@ -503,18 +542,25 @@ int main() {\n\
             print("can't create .\\testbotan.exe", file=sys.stderr)
             sys.exit(1)
         ret = subprocess.call(".\\testbotan.exe")
-        if ret == 1:
+        if ret == 0:
             print("Botan version too old", file=sys.stderr)
             sys.exit(1)
-        if ret == 2:
+        elif ret == 1:
+            botan_version_major = 1
+            botan_version_minor = 10
+        elif ret == 2:
+            botan_version_major = 1
             botan_version_minor = 11
-            print("Botan version 11 not yet supported", file=sys.stderr)
+            print("Botan version 1.11 not yet supported", file=sys.stderr)
             sys.exit(1)
-        if ret != 0:
-            print("Botan test failed", file=sys.stderr)
+        elif ret == 3:
+            botan_version_major = 2
+            botan_version_minor = 0
+            print("Botan version 2.0 not yet supported", file=sys.stderr)
             sys.exit(1)
         else:
-            botan_version_minor = 10
+            print("Botan test failed", file=sys.stderr)
+            sys.exit(1)
 
         # Botan ECC support
         if enable_ecc:
@@ -525,14 +571,20 @@ int main() {\n\
 #include <botan/init.h>\n\
 #include <botan/ec_group.h>\n\
 #include <botan/oids.h>\n\
+#include <botan/version.h>\n\
 int main() {\n\
  Botan::LibraryInitializer::initialize();\n\
  const std::string name("secp256r1");\n\
  const Botan::OID oid(Botan::OIDS::lookup(name));\n\
  const Botan::EC_Group ecg(oid);\n\
  try {\n\
+#if BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(1,11,0)\n\
+  const std::vector<Botan::byte> der =\n\
+   ecg.DER_encode(Botan::EC_DOMPAR_ENC_OID);\n\
+#else\n\
   const Botan::SecureVector<Botan::byte> der =\n\
    ecg.DER_encode(Botan::EC_DOMPAR_ENC_OID);\n\
+#endif\n\
  } catch(...) {\n\
   return 1;\n\
  }\n\
@@ -558,14 +610,20 @@ int main() {\n\
 #include <botan/init.h>\n\
 #include <botan/gost_3410.h>\n\
 #include <botan/oids.h>\n\
+#include <botan/version.h>\n\
 int main() {\n\
  Botan::LibraryInitializer::initialize();\n\
  const std::string name("gost_256A");\n\
  const Botan::OID oid(Botan::OIDS::lookup(name));\n\
  const Botan::EC_Group ecg(oid);\n\
  try {\n\
+#if BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(1,11,0)\n\
+  const std::vector<Botan::byte> der =\n\
+   ecg.DER_encode(Botan::EC_DOMPAR_ENC_OID);\n\
+#else\n\
   const Botan::SecureVector<Botan::byte> der =\n\
    ecg.DER_encode(Botan::EC_DOMPAR_ENC_OID);\n\
+#endif\n\
  } catch(...) {\n\
   return 1;\n\
  }\n\
@@ -592,16 +650,23 @@ int main() {\n\
         print('\
 #include <botan/botan.h>\n\
 #include <botan/rfc3394.h>\n\
+#include <botan/version.h>\n\
+using namespace Botan;\n\
 int main() {\n\
- using namespace Botan;\n\
+#if BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(1,11,0)\n\
+ secure_vector<byte> key(10);\n\
+ SymmetricKey kek("AABB");\n\
+ secure_vector<byte> x = rfc5649_keywrap(key, kek);\n\
+#else\n\
  SecureVector<byte> key(10);\n\
  SymmetricKey kek("AABB");\n\
  Algorithm_Factory& af = global_state().algorithm_factory();\n\
  SecureVector<byte> x = rfc5649_keywrap(key, kek, af);\n\
+#endif\n\
  return 1;\n\
 }', file=testfile)
         testfile.close()
-        command = ["cl", "/nologo", "/MD", "/I", inc, "testrfc5649.cpp", lib]
+        command = ["cl", "/nologo", "/MD", "/EHsc","/I", inc, "testrfc5649.cpp", lib]
         command.extend(system_libs)
         subprocess.call(command)
         if os.path.exists(".\\testrfc5649.exe"):
@@ -613,7 +678,7 @@ int main() {\n\
                 print("can't compile Botan AES key wrap with pad")
 
         # Botan GNU MP support
-        if botan_version_minor == 10:
+        if botan_version_major == 1 and botan_version_minor == 10:
             if verbose:
                 print("checking Botan GNU MP support")
             testfile = open("testgnump.cpp", "w")
@@ -635,6 +700,33 @@ int main() {\n\
                 if verbose:
                     print("Botan GNU MP is not supported")
 
+        # Botan raw PSS support
+        if verbose:
+            print("checking Botan raw PSS support")
+        testfile = open("testrawpss.cpp", "w")
+        print('\
+#include <botan/botan.h>\n\
+#include <botan/version.h>\n\
+int main() {\n\
+#if BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(2,3,0)\n\
+ return 0;\n\
+#endif\n\
+ return 1;\n\
+}', file=testfile)
+        testfile.close()
+        command = ["cl", "/nologo", "/MD", "/I", inc, "testrawpss.cpp", lib]
+        command.extend(system_libs)
+        subprocess.check_output(command, stderr=subprocess.STDOUT)
+        if not os.path.exists(".\\testrawpss.exe"):
+            if verbose:
+                print("can't create .\\testrawpss.exe", file=sys.stderr)
+        else:
+            if subprocess.call(".\\testrawpss.exe") != 0:
+                if verbose:
+                    print("can't find raw PSS: upgrade to Botan >= 2.3.0", file=sys.stderr)
+            else:
+                condvals["RAWPSS"] = True
+
     else:
 
         condvals["OPENSSL"] = True
@@ -651,13 +743,16 @@ int main() {\n\
         if os.path.exists(os.path.join(openssl_lib, "libeay32.lib")):
             openssl_lib_name = "libeay32.lib"
             openssl_lib_dll = "bin\\libeay32.dll"
+        elif os.path.exists(os.path.join(openssl_lib, "libeaycompat32.lib")):
+            openssl_lib_name = "libeaycompat32.lib"
+            openssl_lib_dll = "bin\\libeaycompat32.dll"
         elif os.path.exists(os.path.join(openssl_lib, "libcrypto.lib")):
             openssl_lib_name = "libcrypto.lib"
             if platform == 32:
                 openssl_lib_dll = "bin\\libcrypto-1_1.dll"
             else:
                 openssl_lib_dll = "bin\\libcrypto-1_1-x64.dll"
-                
+
         else:
             print("can't find OpenSSL library", file=sys.stderr)
             sys.exit(1)
@@ -784,7 +879,7 @@ int main() {\n\
  ENGINE *eg;\n\
  const EVP_MD* EVP_GOST_34_11;\n\
  OpenSSL_add_all_algorithms();\n\
-#if OPENSSL_VERSION_NUMBER < 0x10100000L\n\
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)\n\
  ENGINE_load_builtin_engines();\n\
 #else\n\
  OPENSSL_init_crypto(OPENSSL_INIT_ENGINE_ALL_BUILTIN | OPENSSL_INIT_LOAD_CONFIG, NULL);\n\
@@ -858,7 +953,12 @@ int main() {\n\
         else:
             if verbose:
                 print("can't compile OpenSSL RFC 5649")
-        
+
+        # no check for OpenSSL raw PSS support
+        condvals["RAWPSS"] = True
+        # no check for OpenSSL AES GCM
+        condvals["AESGCM"] = True
+
     # configure CppUnit
     if want_tests:
         condvals["TESTS"] = True
@@ -1022,8 +1122,8 @@ def main(args):
         if want_tests:
             print("cppunit-path: " + cppunit_path)
         print("toolset: "+toolset)
-    
-    
+
+
     doconfig()
 
     # status after config
